/*
 * Copyright (c) 2015 - 2016 , Freescale Semiconductor, Inc.
 * Copyright 2016-2018 NXP
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NXP "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NXP OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/* ###################################################################
**     Filename    : main.c
**     Project     : lpuart_echo_s32k144
**     Processor   : S32K144_100
**     Version     : Driver 01.00
**     Compiler    : GNU C Compiler
**     Date/Time   : 2017-08-04, 12:27, # CodeGen: 1
**     Abstract    :
**         Main module.
**         This module contains user's application code.
**     Settings    :
**     Contents    :
**         No public methods
**
** ###################################################################*/
/*!
** @file main.c
** @version 01.00
** @brief
**         Main module.
**         This module contains user's application code.
*/
/*!
**  @addtogroup main_module main module documentation
**  @{
*/
/* MODULE main */

/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "pin_mux.h"
#include "dmaController1.h"
#include "clockMan1.h"
#include "lpuart1.h"
#include "FTM2.h"
#include "sent_encode.h"

volatile int exit_code = 0;
/* User includes (#include below this line is not maintained by Processor Expert) */
#include <string.h>
#include <stdint.h>
#include <stdbool.h>

/* Welcome message displayed at the console */
#define welcomeMsg "Please type the number with the action you want to perform and press Enter:\r\n\
1: Transmit Sent message/s\r\n\
2: Change Settings\r\n"

#define txMessage "Type or open a file with the sent message/s (max. 64 Messages) you want to send and press Enter\r\n\
Format: 1 nibble Status+ 6 nibble data (separator between messages '-') (CRC is added automatically) Example 2 messages: 12AF347-02D490F\r\n\
:"

/* Error message displayed at the console, in case data is received erroneously */
#define errorMsg "An error occurred! The application will stop!\r\n"

/* Timeout in ms for blocking operations */
#define TIMEOUT     1000U

/* Receive buffer size */
#define BUFFER_SIZE 512U

#define SENT_MESSAGE_LENGTH 7u // 6 Data + Status
#define SENT_VALID_MESSAGE 0u
#define SENT_NO_VALID_MESSAGE 1u

/* Buffer used to receive data from the console */
uint8_t buffer[BUFFER_SIZE];
uint8_t bufferIdx;
uint8_t *gpBuffer;

uint32_t prepareSentMessage(uint8_t *pBuffer);

/* UART rx callback for continuous reception, byte by byte */
void rxCallback(void *driverState, uart_event_t event, void *userData)
{
    /* Unused parameters */
    (void)driverState;
    (void)userData;

    /* Check the event type */
    if (event == UART_EVENT_RX_FULL)
    {
        /* The reception stops when newline is received or the buffer is full */
        if ((buffer[bufferIdx] != '\n') && (bufferIdx != (BUFFER_SIZE - 2U)))
        {
            /* Update the buffer index and the rx buffer */
            bufferIdx++;
            LPUART_DRV_SetRxBuffer(INST_LPUART1, &buffer[bufferIdx], 1U);
        }
    }
}

/*!
 \brief The main function for the project.
 \details The startup initialization sequence is the following:
 * - startup asm routine
 * - main()
 */
int main(void)
{
  /* Write your local variable definition here */
  status_t status;
  /* Declare a buffer used to store the received data */
  uint32_t bytesRemaining;

  /*** Processor Expert internal initialization. DON'T REMOVE THIS CODE!!! ***/
#ifdef PEX_RTOS_INIT
  PEX_RTOS_INIT(); /* Initialization of the selected RTOS. Macro is defined by the RTOS component. */
#endif
  /*** End of Processor Expert internal initialization.                    ***/

  /* Write your code here */
  /* For example: for(;;) { } */

  /* Initialize and configure clocks
   *     -    see clock manager component for details
   */
  CLOCK_SYS_Init(g_clockManConfigsArr, CLOCK_MANAGER_CONFIG_CNT,
                 g_clockManCallbacksArr, CLOCK_MANAGER_CALLBACK_CNT);
  CLOCK_SYS_UpdateConfiguration(0U, CLOCK_MANAGER_POLICY_AGREEMENT);

  /* Initialize pins
   *    -    See PinSettings component for more info
   */
  PINS_DRV_Init(NUM_OF_CONFIGURED_PINS, g_pin_mux_InitConfigArr);

  FTM2_init();
  FTM2_CH0_OC_init();
  DMA_init();
  DMA_TCD_init();



  /* Initialize LPUART instance */
  LPUART_DRV_Init(INST_LPUART1, &lpuart1_State, &lpuart1_InitConfig0);
  /* Install the callback for rx events */
  LPUART_DRV_InstallRxCallback(INST_LPUART1, rxCallback, NULL);
  /* Send a welcome message */
  LPUART_DRV_SendDataBlocking(INST_LPUART1, (uint8_t *)welcomeMsg, strlen(welcomeMsg), TIMEOUT);

  /* Infinite loop:
   *     - Receive data from user
   *     - Echo the received data back
   */
  while (1)
  {
      /* Receive and store data byte by byte until new line character is received,
       * or the buffer becomes full (256 characters received)
       */
      LPUART_DRV_ReceiveData(INST_LPUART1, buffer, 1U);
      /* Wait for transfer to be completed */
      while(LPUART_DRV_GetReceiveStatus(INST_LPUART1, &bytesRemaining) == STATUS_BUSY);

      /* Check the status */
      status = LPUART_DRV_GetReceiveStatus(INST_LPUART1, &bytesRemaining);

      if (status != STATUS_SUCCESS)
      {
          /* If an error occurred, send the error message and exit the loop */
          LPUART_DRV_SendDataBlocking(INST_LPUART1, (uint8_t *)errorMsg, strlen(errorMsg), TIMEOUT);
          break;
      }

      /* Append string terminator to the received data */
      bufferIdx++;
      buffer[bufferIdx] = 0U;

      /* If the received string is "Hello Board", send back "Hello World" */
      if(strcmp((char *)buffer, "1\r\n") == 0)
      {

        LPUART_DRV_SendDataBlocking(INST_LPUART1, (uint8_t *)txMessage, strlen(txMessage), TIMEOUT);

        bufferIdx = 0;

        LPUART_DRV_ReceiveData(INST_LPUART1, buffer, 1U);
        /* Wait for transfer to be completed */
        while(LPUART_DRV_GetReceiveStatus(INST_LPUART1, &bytesRemaining) == STATUS_BUSY);

        /* Check the status */
        status = LPUART_DRV_GetReceiveStatus(INST_LPUART1, &bytesRemaining);

        if (status != STATUS_SUCCESS)
        {
            /* If an error occurred, send the error message and exit the loop */
            LPUART_DRV_SendDataBlocking(INST_LPUART1, (uint8_t *)errorMsg, strlen(errorMsg), TIMEOUT);
            break;
        }
        gpBuffer = buffer;

        if (prepareSentMessage(gpBuffer) == SENT_VALID_MESSAGE)
        {
          gpBuffer += 8;
          start_FTM2_counter();
        }

        LPUART_DRV_SendDataBlocking(INST_LPUART1, (uint8_t *)"Done!\r\nType:", strlen("Done!\r\nType:"), TIMEOUT);
      }
      else if(strcmp((char *)buffer, "2\r\n") == 0)
      {
        LPUART_DRV_SendDataBlocking(INST_LPUART1, (uint8_t *)"Not yet implemented!\r\nType:", strlen("Not yet implemented!"), TIMEOUT);
      }

      bufferIdx = 0;

  }

  /*** Don't write any code pass this line, or it will be deleted during code generation. ***/
  /*** RTOS startup code. Macro PEX_RTOS_START is defined by the RTOS component. DON'T MODIFY THIS CODE!!! ***/
  #ifdef PEX_RTOS_START
    PEX_RTOS_START();                  /* Startup of the selected RTOS. Macro is defined by the RTOS component. */
  #endif
  /*** End of RTOS startup code.  ***/
  /*** Processor Expert end of main routine. DON'T MODIFY THIS CODE!!! ***/
  for(;;) {
    if(exit_code != 0) {
      break;
    }
  }
  return exit_code;
  /*** Processor Expert end of main routine. DON'T WRITE CODE BELOW!!! ***/
} /*** End of main routine. DO NOT MODIFY THIS TEXT!!! ***/

/* END main */

void DMA0_IRQHandler(void)
{
  DMA->INT |= 1;
  stop_FTM2_counter();
  //FTM2_init();
  FTM2_CH0_OC_init();
  if (prepareSentMessage(gpBuffer) == SENT_VALID_MESSAGE)
  {
    gpBuffer += 8;
    start_FTM2_counter();
  }
}


uint32_t prepareSentMessage(uint8_t *pBuffer)
{
  uint32_t ret = SENT_VALID_MESSAGE;

  for(int i=0;i<SENT_MESSAGE_LENGTH;i++)
  {
    switch (pBuffer[i])
    {
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
        sentEncodeNextNibble(pBuffer[i] - 48u);
        break;
      case 'A':
      case 'a':
        sentEncodeNextNibble(0x0Au);
        break;

      case 'B':
      case 'b':
        sentEncodeNextNibble(0x0Bu);
        break;

      case 'C':
      case 'c':
        sentEncodeNextNibble(0x0Cu);
        break;

      case 'D':
      case 'd':
        sentEncodeNextNibble(0x0Du);
        break;
      case 'E':
      case 'e':
        sentEncodeNextNibble(0x0Eu);
        break;

      case 'F':
      case 'f':
        sentEncodeNextNibble(0x0Fu);
        break;

      default:
        ret = SENT_NO_VALID_MESSAGE;
        break;
    }
  }

  return ret;
}
/*!
 ** @}
 */
/*
 ** ###################################################################
 **
 **     This file was created by Processor Expert 10.1 [05.21]
 **     for the Freescale S32K series of microcontrollers.
 **
 ** ###################################################################
 */

